	package eu.etaxonomy.taxeditor.prototype2.controller;

import org.eclipse.jface.action.Action;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.Activator;
import eu.etaxonomy.taxeditor.prototype2.model.NameEditorInput;
import eu.etaxonomy.taxeditor.prototype2.view.MultiPageTaxonView;
import eu.etaxonomy.taxeditor.prototype2.view.NameListView;
import eu.etaxonomy.taxeditor.prototype2.view.TaxonomicTreeViewer;

/**
 * Opens a name editor for an existing or new taxon
 * 
 * @author p.ciardelli
 *
 */
public class ActionOpenNameEditor extends Action {

	private Taxon taxon;
	public static final String ID = "eu.etaxonomy.taxeditor.prototype2.controller.opennameeditoraction"; //$NON-NLS-1$
	
	public ActionOpenNameEditor(Taxon taxon) {
		this.taxon = taxon;
	}

	public ActionOpenNameEditor() {

		// the text for the menu item
		setText("New taxon");
		
		setId(ID);
		
		this.taxon = new Taxon();
		taxon.setName(new BotanicalName(Rank.GENUS()));
		
	}
	
	public void run() {

		// If this is a non-empty taxon being opened for the first time,
		//  add to recent names list		
		if (taxon.getName().getTitleCache() != null &&
				!Activator.getDefault().getObservableRecentNamesList().contains(taxon))
			Activator.getDefault().getObservableRecentNamesList().add(0, taxon);
		
		// If this taxon is not visible in the tree, open node
		((TaxonomicTreeViewer) ((NameListView) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().
				findView(NameListView.ID)).getTreeViewer()).revealTaxon(taxon);
				
		IEditorInput input = new NameEditorInput(taxon);
        	try {
				Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().
					getActivePage().openEditor(input, MultiPageTaxonView.ID);
			} catch (PartInitException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
        
	}
}