// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.reference;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;

/**
 * @author p.ciardelli
 * @created 07.04.2009
 * @version 1.0
 */
public class ReferenceSelectComposite extends Composite {
	private static final Logger logger = Logger
			.getLogger(ReferenceSelectComposite.class);
	private ReferenceBase savedReference;
	private Text txtReference;
	private Button btnClearReference;
	private Text txtMicroRef;
	
	/**
	 * @param parent
	 * @param style
	 */
	public ReferenceSelectComposite(Composite parent) {
		super(parent, SWT.NULL);
		createContent(parent);
	}
	
	private void createContent(Composite xcontainer) {
		setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		GridLayout gridLayout = new GridLayout();
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		Label lblReference = new Label(this, SWT.NONE);
		lblReference.setLayoutData(new GridData());
		lblReference.setText("Choose a reference either by searching or entering it as free text:");
		
		// Create 3-columned composite for reference input
		Composite refComposite = new Composite(this, SWT.NONE);
		GridLayout gridLayout2 = new GridLayout();
		gridLayout2.numColumns = 3;	
		gridLayout2.marginHeight = 0;
		gridLayout2.marginWidth = 0;
		refComposite.setLayout(gridLayout2);
		refComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		// Create reference input
		txtReference = new Text(refComposite, SWT.BORDER);
		txtReference.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		// Create reference search button
		Button btnSearchReference = new Button(refComposite, SWT.NONE);
		btnSearchReference.setEnabled(true);
		btnSearchReference.setLayoutData(new GridData());
		btnSearchReference.setText("Search ...");
		btnSearchReference.addSelectionListener(new SelectionAdapter() {
			
			// Popup reference search
			public void widgetSelected(SelectionEvent e) {
				popupSearch();
			}
		});

		// Create clear reference button
		btnClearReference = new Button(refComposite, SWT.NONE);
		btnClearReference.setEnabled(false);
		btnClearReference.setText("Clear");
		btnClearReference.addSelectionListener(new SelectionAdapter() {
			
			// Clear selected reference
			public void widgetSelected(SelectionEvent e) {
				clearReference();
			}
		});
		
		// Create message re: editing references
		Label lblNewRefFeedback = new Label(this, SWT.NONE);
		lblNewRefFeedback.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false));
		lblNewRefFeedback.setFont(
				TaxEditorPlugin.getDefault().italicizeFont(lblNewRefFeedback.getFont()));
		lblNewRefFeedback.setText("Existing references can only be edited in property sheet.");
				
		Label lblMicroref = new Label(this, SWT.NONE);
		lblMicroref.setText("Enter reference detail (e.g., page no.):");
		
		// Create microref input
		txtMicroRef = new Text(this, SWT.BORDER);
		txtMicroRef.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false));
	}
	
	public void setReference(ReferenceBase reference) {
		savedReference = reference;
		txtReference.setText(reference.getTitleCache());
		txtReference.setEditable(false);
					
		btnClearReference.setEnabled(true);
	}
	
	public void setMicroReference(String microref) {
		txtMicroRef.setText(microref);
	}
	
	/**
	 * 
	 */
	protected void popupSearch() {
		Dialog dialog = new ReferenceSearchDialog(getShell(), 
				IReferenceSearch.BIBREF);
        Object value = ((ReferenceSearchDialog) dialog).open();
        
		if (value instanceof ReferenceBase) {
			setSavedReference((ReferenceBase) value);
		}
	}

	/**
	 * @param value
	 */
	private void setSavedReference(ReferenceBase reference) {
		
		savedReference = reference;
		
		txtReference.setText(reference.getTitleCache());
		txtReference.setEditable(false);
		
		btnClearReference.setEnabled(true);
	}
	
	protected void clearReference() {
		savedReference = null;
		
		txtReference.setText("");
		txtReference.setEditable(true);
		
		btnClearReference.setEnabled(false);		
	}

	/**
	 * Returns reference object, if any. Otherwise, creates a new Generic
	 * reference object using text the user has i nput.
	 *  
	 * @return
	 */
	public ReferenceBase getReference() {
		ReferenceBase reference = null;
	    if (savedReference != null) {
	    	reference = savedReference;
		} else {
			if (!txtReference.getText().equals("")) {
				reference = Generic.NewInstance();
				reference.setTitleCache(txtReference.getText());
			}
		}
		return reference;
	}
	
	public String getMicroReference() {
		return txtMicroRef.getText();
	}
}