/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.propertysheet.namerelations.wizard;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.actions.cdm.CreateNameRelationAction;

/**
 * @author p.ciardelli
 * @created 04.06.2008
 * @version 1.0
 */
public class NameRelationWizard extends Wizard {
	private static final Logger logger = Logger
			.getLogger(NameRelationWizard.class);
	
	public static final String ID = "eu.etaxonomy.taxeditor.namerelationwizard";

	private NameRelationWizardModel relationModel;

	public NameRelationWizard(NameRelationWizardModel relationModel) {
		super();
		
		this.relationModel = relationModel;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	public void addPages() {
		IWizardPage chooseNamePage = new ChooseRelatedNameWizardPage(relationModel);
    	addPage(chooseNamePage);
    	IWizardPage chooseRelationPage = new ChooseRelationTypeWizardPage(relationModel);
    	addPage(chooseRelationPage);
    }
	
	@Override
	public boolean performFinish() {
		CreateNameRelationAction nameRelationAction = new CreateNameRelationAction(relationModel.getFromName(),
				relationModel.getType(),
				relationModel.getToName());
//		if (relationModel.getNameRelationsList() != null) {
			
			// Action will notify when relation has been created 
			nameRelationAction.addPropertyChangeListener(new IPropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent event) {
					logger.warn(event.getProperty());
					if (event.getProperty().equals(
							ITaxEditorConstants.NAMERELATION)) {
						if (relationModel.getNameRelationsList() != null) {
							relationModel.getNameRelationsList().clear();
							relationModel.getNameRelationsList().addAll(relationModel.getBaseName().getNameRelations());
						}
					}
					
				}
			});
//		}
		nameRelationAction.run();
		return true;
	}
}
