// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.MessageDialogWithToggle;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.AnnotationModel;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.views.properties.IPropertySheetEntry;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetSorter;

import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineDocumentProvider;
import eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineEditor;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotation;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotationModel;
import eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author p.ciardelli
 * @created 07.07.2009
 * @version 1.0
 */
public class BulkEditor extends AnnotatedLineEditor implements PropertyChangeListener {
	private static final Logger logger = Logger.getLogger(BulkEditor.class);

	public static final String ID = "bulkeditor.editor";

	private boolean isInitialFocus = true;

	private IPropertyChangeListener markerPreferenceListener;
	
	/**
	 * 
	 */
	public BulkEditor() {
		super(CdmStore.NewTransactionalConversation());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.editors.text.TextEditor#initializeEditor()
	 */
	@Override
	protected void initializeEditor() {
		super.initializeEditor();
				
		/**
		 * see AbstractTextEditor javadoc for explanation of context menu ids
		 */
		setEditorContextMenuId("#BulkEditorContext");

		setEntityCreatorService(new BulkEditorEntityCreatorService());
		
		setPersistenceService(new BulkEditorPersistenceService());
		
		setLineDisplayStrategy(new BulkEditorLineDisplay(this));
		
		setSourceViewerConfiguration(new BulkEditorViewerConfiguration(lineDisplayStrategy));
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ListEditor#createSourceViewer(org.eclipse.swt.widgets.Composite, org.eclipse.jface.text.source.IVerticalRuler, int)
	 */
	protected ISourceViewer createSourceViewer(Composite parent,
			IVerticalRuler ruler, int styles) {
		ISourceViewer viewer = super.createSourceViewer(parent, ruler, styles);
		if (((BulkEditorInput) getEditorInput()).isMergingEnabled()) {
			addToggleMergeCandidateListener(ruler.getControl());
		}
		return viewer;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.AnnotatedLineEditor#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 */
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		
		if (!(input instanceof BulkEditorInput)) {
			throw new PartInitException("Invalid Input: Must be BulkEditorInput");
		}
		
		super.init(site, input);
		
		((BulkEditorInput) input).addPropertyChangeListener(this);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {
		
		GridLayout gridLayout = new GridLayout();
		parent.setLayout(gridLayout);
		
		BulkEditorSearchComposite searchBar = new BulkEditorSearchComposite(this, parent, SWT.NONE);
		searchBar.getParent().setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));

		if (isEditableMarkerTypes()) {
			createMarkerEditing(parent);
		}
		
		super.createPartControl(parent);
		
		// Set viewer composite to fill grid
		for (Control control : parent.getChildren()) {
			if (control instanceof Composite && 
						!(control instanceof BulkEditorSearchComposite || control instanceof BulkEditorMarkerPreferenceComposite)) {
				control.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
			}
		}
	}
	
	/**
	 * @param parent 
	 * 
	 */
	private void createMarkerEditing(Composite parent) {
		final BulkEditorMarkerPreferenceComposite markerPreferenceBar = 
				new BulkEditorMarkerPreferenceComposite((BulkEditorInput) getEditorInput(), parent, SWT.NONE);
		markerPreferenceListener = new IPropertyChangeListener() {
			public void propertyChange(
					org.eclipse.jface.util.PropertyChangeEvent event) {
				if (event.getProperty().contains(PreferencesUtil.EDIT_MARKER_TYPE_PREFIX)) {
					refreshLineDisplay();
				}
			}
		};
		PreferencesUtil.getPreferenceStore().addPropertyChangeListener(markerPreferenceListener);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.editors.text.TextEditor#dispose()
	 */
	@Override
	public void dispose() {
		if (markerPreferenceListener != null ) {
			PreferencesUtil.getPreferenceStore().removePropertyChangeListener(markerPreferenceListener);
		}
		super.dispose();
	}
	
	/**
	 * Can this input type edit any markers?
	 * 
	 * @return
	 */
	private boolean isEditableMarkerTypes() { 
		for (MarkerType markerType : VocabularyStore.getNonTechnicalMarkerTypes()) {
			if (((BulkEditorInput) getEditorInput()).isMarkerTypeEditingEnabled(markerType)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Refreshes text in all lines.
	 */
	protected void refreshLineDisplay() {
		IDocumentProvider provider = getDocumentProvider();
		IEditorInput input = getEditorInput();
		IAnnotationModel model = provider.getAnnotationModel(input);
		Iterator iter = getDocumentProvider().getAnnotationModel(getEditorInput()).getAnnotationIterator();
		while (iter.hasNext()) {
			Object next = iter.next();
			if (next instanceof LineAnnotation) {
				LineAnnotation annotation = (LineAnnotation) next;
				((AnnotatedLineDocumentProvider) getDocumentProvider()).
						updateLineFromAnnotation(getEditorInput(), annotation);
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#isEditable()
	 */
	@Override
	public boolean isEditable() {
		return false;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineEditor#setFocus()
	 */
	@Override
	public void setFocus() {
		super.setFocus();

		// TODO find a better place to put this - this dialog should be shown after initial contents of 
		//	Editor are displayed
		if (isInitialFocus) {
			displayWarningDialog();
			isInitialFocus = false;
		}
	}
	
	/**
	 * 
	 */
	private void displayWarningDialog() {
		IPreferenceStore prefs = PreferencesUtil.getPreferenceStore();
		if (!prefs.getBoolean(PreferencesUtil.HIDE_BULKEDITOR_INFO)) {
			String msg = "The Bulk Editor allows you to edit objects used to reference other objects, such as names, references, and authors.\n\n" +
							"Any changes you make to an object in the Bulk Editor will be displayed wherever the object is used.\n\n" +
							"For instance, a reference may be displayed with both a name and a descriptive element. If the reference name is changed here, the display of both the name and the descriptive element will be affected.";
			MessageDialogWithToggle dialog = MessageDialogWithToggle.openOkCancelConfirm
												(getSite().getShell(), "Bulk Editor", msg, "Do not show this message again", 
														false, null, PreferencesUtil.HIDE_BULKEDITOR_INFO);
			if (dialog.getReturnCode() == Window.OK) { 
				prefs.setValue(PreferencesUtil.HIDE_BULKEDITOR_INFO, dialog.getToggleState());
			}
		}
	}
	
	/**
	 * @param control
	 */
	private void addToggleMergeCandidateListener(Control control) {
		control.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDoubleClick(MouseEvent e) {
				StyledText textWidget = getSourceViewer().getTextWidget();
				int line = textWidget.getLineIndex(e.y);
				toggleMergeCandidateAnnotation(line);
			}
		});
	}

	/**
	 * @param line
	 */
	public void toggleMergeCandidateAnnotation(int line) {
		
		IDocument document = getSourceViewer().getDocument();
		LineAnnotationModel model = 
				(LineAnnotationModel) getSourceViewer().getAnnotationModel();
		
		Annotation annotation = model.getAnnotationAtLine(line, document);
		
		if (annotation != null) {
			if (annotation.getType().equals(IBulkEditorConstants.TYPE_MERGE_CANDIDATE)) {
				model.changeAnnotationType(
						annotation, LineAnnotation.TYPE_GENERIC);
			} else {
				model.changeAnnotationType(
						annotation, IBulkEditorConstants.TYPE_MERGE_CANDIDATE);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(BulkEditorInput.QUERY_CHANGED)) {
			
			// TODO check if dirty, prompt save
			if (isDirty()) {
				boolean proceed = MessageDialog.openQuestion(getEditorSite().getShell(), 
						"Save changes", "You have made changes that must be saved before this query can be executed. Would you like to proceed?");
				if (proceed) {
					doSave(null);
				} else {
					return;
				}
			}
			((AnnotationModel) getDocumentProvider().getAnnotationModel(getEditorInput())).removeAllAnnotations();
			setInput(getEditorInput());
		}
	}
	
	@SuppressWarnings("unchecked")
	public Object getAdapter(Class type) {
	    if (type == IPropertySheetPage.class) {
	    	PropertySheetPage page = new EditorPropertySheetPage();
	        return page;	    	
	    }
	    return super.getAdapter(type);
	}
	
	class EditorPropertySheetPage extends PropertySheetPage {
		
		EditorPropertySheetPage() {
			super();
			
			// Override sorter to simply display names as first-in-first-out
			setSorter(new PropertySheetSorter() {
				public int compare(IPropertySheetEntry entryA, IPropertySheetEntry entryB) {
					return 0;
				}
				public int compareCategories(String categoryA, String categoryB) {
					return 0;
				}
				public void sort(IPropertySheetEntry[] entries) {
					// do nothing
				}
			});
		}
		
        public void makeContributions(IMenuManager menuManager,
                IToolBarManager toolBarManager, IStatusLineManager statusLineManager) {
        	super.makeContributions(menuManager, toolBarManager, statusLineManager);
        
        	// Remove "Show categories", "Show advanced properties", "Restore default value"
        	toolBarManager.removeAll();
        	menuManager.removeAll();
        }
	}
}