// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;

import eu.etaxonomy.taxeditor.annotatedlineeditor.SupplementalTextScanner;
import eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy;


/**
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class BulkEditorViewerConfiguration extends SourceViewerConfiguration {
	private static final Logger logger = Logger
			.getLogger(BulkEditorViewerConfiguration.class);
	
	private ILineDisplayStrategy lineDisplayStrategy;

	public BulkEditorViewerConfiguration(ILineDisplayStrategy lineDisplayStrategy) {
		this.lineDisplayStrategy = lineDisplayStrategy;
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getPresentationReconciler(org.eclipse.jface.text.source.ISourceViewer)
	 */
	@Override
	public IPresentationReconciler getPresentationReconciler(
			final ISourceViewer sourceViewer) {
		
		PresentationReconciler reconciler = new PresentationReconciler();
		
		// Color supplemental text, e.g. markers
		DefaultDamagerRepairer damageRepairer = new DefaultDamagerRepairer(new SupplementalTextScanner(sourceViewer, lineDisplayStrategy));
		reconciler.setDamager(damageRepairer, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(damageRepairer, IDocument.DEFAULT_CONTENT_TYPE);
		
		return reconciler;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getConfiguredContentTypes(org.eclipse.jface.text.source.ISourceViewer)
	 */
	@Override
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] { IDocument.DEFAULT_CONTENT_TYPE, ILineDisplayStrategy.SUPPLEMENTAL_TEXT };
	}
}