/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree.detailpage;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.IDetailsPage;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.SharedScrolledComposite;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.taxeditor.forms.EditorFormToolkit;
import eu.etaxonomy.taxeditor.forms.entitysections.AnnotationSection;
import eu.etaxonomy.taxeditor.forms.entitysections.OriginalSourceSection;

/**
 * @author p.ciardelli
 * @author n.hoffmann
 *
 */
public abstract class AbstractDescriptionDetailsPage implements IDetailsPage, IPropertyChangeListener {	
	private static final Logger logger = Logger.getLogger(AbstractDescriptionDetailsPage.class);

	protected IManagedForm managedForm;
	protected Object input;
	protected Section section;
	protected Composite client;
	protected EditorFormToolkit toolkit;
	
	private boolean hasCitations = false;
	private boolean hasAnnotations = false;
	private boolean hasExtensions = false;
		
	protected AbstractDescriptionDetailSection descriptionSection;
	private OriginalSourceSection citationSection;
	private AnnotationSection annotationSection;
	private ExtensionSection extensionSection;

	protected Composite parent;

	/**
	 * 
	 */
	abstract void configurePage();

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public void createContents(Composite parent) {

		// store the parent
		this.parent = parent;
		
		configurePage();
		
		toolkit = new EditorFormToolkit(managedForm.getToolkit().getColors());
		
		parent.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		parent.setLayout(new GridLayout());
		
		createDescriptionContents(parent);
		
		if(hasCitations)
			citationSection = toolkit.createOriginalSourceSection(parent, getDescriptionElement(), SWT.NULL);
			if (citationSection != null)
				citationSection.addPropertyChangeListener(this);
		if(hasAnnotations)
			annotationSection = toolkit.createAnnotationSection(parent, getDescriptionElement(), SWT.NULL);
			if(annotationSection != null)
				annotationSection.addPropertyChangeListener(this);
		if(hasExtensions)
			// there are no extensions for description elements
		
		reflow(true);
	}
	
	/**
	 * Reflows the shared scrolled composite of this description details page. Has to be called 
	 * every time controls are added or removed from a composite in this page to ensure correct display
	 * of the controls.
	 * 
	 * @param changed if <code>true</code>, drop the cached data
	 */
	public void reflow(boolean changed){
		((SharedScrolledComposite) parent.getParent().getParent()).reflow(changed);
	}
	
	/**
	 * 
	 * @param parent
	 */
	abstract protected void createDescriptionContents(Composite parent);	

	/* (non-Javadoc)
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		managedForm.dirtyStateChanged();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#commit(boolean)
	 */
	public void commit(boolean onSave) {
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#dispose()
	 */
	public void dispose() {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#initialize(org.eclipse.ui.forms.IManagedForm)
	 */
	public void initialize(IManagedForm form) {
		this.managedForm = form;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#isDirty()
	 */
	public boolean isDirty() {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#isStale()
	 */
	public boolean isStale() {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#refresh()
	 */
	public void refresh() {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#setFocus()
	 */
	public void setFocus() {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#setFormInput(java.lang.Object)
	 */
	public boolean setFormInput(Object input) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IPartSelectionListener#selectionChanged(org.eclipse.ui.forms.IFormPart, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IFormPart part, ISelection selection) {	
		IStructuredSelection structuredSelection = (IStructuredSelection)selection;
		if (structuredSelection.size() == 1) {
			input = structuredSelection.getFirstElement();
		} else {
			input = null;
		}
		
		updateSections();
	}

	/**
	 * Update the details Page sections with the new selection
	 */
	private void updateSections(){
		descriptionSection.updateSection();
				
		if(hasCitations){
			citationSection.setEntity(getDescriptionElement());
		}
		if(hasAnnotations){
			annotationSection.setEntity(getDescriptionElement());
		}
		if(hasExtensions){
//			extensionSection.updateSection();
		}
	}

	protected Label createLabel(String string, Composite composite, Composite parent) {
		Label label = new Label(composite, SWT.WRAP);
        label.setText(string);
        GridData data = new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_CENTER);
        label.setLayoutData(data);
        label.setFont(parent.getFont());
        return label;
	}

	/**
	 * @return the hasCitations
	 */
	public boolean hasCitations() {
		return hasCitations;
	}

	/**
	 * @param hasCitations the hasCitations to set
	 */
	public void setHasCitations(boolean hasCitations) {
		this.hasCitations = hasCitations;
	}

	/**
	 * @return the hasAnnotations
	 */
	public boolean hasAnnotations() {
		return hasAnnotations;
	}

	/**
	 * @param hasAnnotations the hasAnnotations to set
	 */
	public void setHasAnnotations(boolean hasAnnotations) {
		this.hasAnnotations = hasAnnotations;
	}

	/**
	 * @return the hasExtensions
	 */
	public boolean hasExtensions() {
		return hasExtensions;
	}

	/**
	 * @param hasExtensions the hasExtensions to set
	 */
	public void setHasExtensions(boolean hasExtensions) {
		this.hasExtensions = hasExtensions;
	}

	/**
	 * @return
	 */
	public DescriptionElementBase getDescriptionElement() {
		return (DescriptionElementBase) input;
	}
}
