/**
 * 
 */
package eu.etaxonomy.taxeditor.forms;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.ToggleHyperlink;

/**
 * @author nho
 *
 */
public abstract class AbstractEditorFormSection extends Section implements IExpansionListener, IPropertyChangeEmitter{


	protected EditorFormToolkit toolkit;
	private Composite client;
	
	private Set<IPropertyChangeListener> propertyChangeListeners = new HashSet<IPropertyChangeListener>();

	/**
	 * @param parent
	 * @param style
	 */
	protected AbstractEditorFormSection(Composite parent, int style) {
		super(parent, Section.TITLE_BAR | SWT.NO_BACKGROUND | style);
		
		toolkit = new EditorFormToolkit(parent.getDisplay());
		
		toolkit.adapt(this);
		
		this.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		client = toolkit.createComposite(this, SWT.WRAP);
		client.setLayout(new GridLayout());

		this.setClient(client);
		this.addExpansionListener(this);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Widget#dispose()
	 */
	@Override
	public void dispose() {
		super.dispose();
		toolkit.dispose();
	}
	
	public ToggleHyperlink getToggle(){
		return this.toggle;
	}
	
	public Composite getClient(){
		return client;
	}

	public AbstractEditorFormSection getSection(){
		return this;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.events.IExpansionListener#expansionStateChanged(org.eclipse.ui.forms.events.ExpansionEvent)
	 */
	public void expansionStateChanged(ExpansionEvent e) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.events.IExpansionListener#expansionStateChanging(org.eclipse.ui.forms.events.ExpansionEvent)
	 */
	public void expansionStateChanging(ExpansionEvent e) {
		// TODO Auto-generated method stub
		
	}	
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IPropertyChangeEmitter#addPropertyChangeListener(org.eclipse.jface.util.IPropertyChangeListener)
	 */
	public void addPropertyChangeListener(IPropertyChangeListener listener) {
		propertyChangeListeners.add(listener);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IPropertyChangeEmitter#removePropertyChangeListener(org.eclipse.jface.util.IPropertyChangeListener)
	 */
	public void removePropertyChangeListener(IPropertyChangeListener listener) {
		propertyChangeListeners.remove(listener);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IPropertyChangeEmitter#firePropertyChangeEvent()
	 */
	public void firePropertyChangeEvent(PropertyChangeEvent event) {
		for(IPropertyChangeListener listener : propertyChangeListeners){
			listener.propertyChange(event);
		}
	}
}
