/**
 * 
 */
package eu.etaxonomy.taxeditor.forms;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.FormToolkit;

/**
 * @author nho
 *
 */
public abstract class AbstractFormComposite extends Composite implements IPropertyChangeEmitter {

	protected FormToolkit toolkit;
	private Set<IPropertyChangeListener> propertyChangeListeners = new HashSet<IPropertyChangeListener>();

	
	/**
	 * @param parent
	 * @param style
	 */
	public AbstractFormComposite(Composite parent, int style) {
		super(parent, style);

		GridLayout layout = new GridLayout();
		layout.marginWidth = 0;
		layout.marginHeight = 0;
		layout.verticalSpacing = 0;
		layout.horizontalSpacing = 0;
		
		this.setLayout(layout);
		
		toolkit = new FormToolkit(parent.getDisplay());
		
		GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		
		this.setLayoutData(gridData);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Widget#dispose()
	 */
	@Override
	public void dispose() {
		super.dispose();
		toolkit.dispose();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IPropertyChangeEmitter#addPropertyChangeListener(org.eclipse.jface.util.IPropertyChangeListener)
	 */
	public void addPropertyChangeListener(IPropertyChangeListener listener) {
		propertyChangeListeners.add(listener);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IPropertyChangeEmitter#removePropertyChangeListener(org.eclipse.jface.util.IPropertyChangeListener)
	 */
	public void removePropertyChangeListener(IPropertyChangeListener listener) {
		propertyChangeListeners.remove(listener);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IPropertyChangeEmitter#firePropertyChangeEvent()
	 */
	public void firePropertyChangeEvent(PropertyChangeEvent event) {
		for(IPropertyChangeListener listener : propertyChangeListeners){
			listener.propertyChange(event);
		}
	}
}
