/**
 * 
 */
package eu.etaxonomy.taxeditor.forms;

import java.net.URI;

import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.IFormColors;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.internal.forms.widgets.FormFonts;

import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.DescriptionElementSource;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.forms.entitysections.AnnotationComposite;
import eu.etaxonomy.taxeditor.forms.entitysections.AnnotationSection;
import eu.etaxonomy.taxeditor.forms.entitysections.DescriptionElementSourceComposite;
import eu.etaxonomy.taxeditor.forms.entitysections.OriginalSourceSection;
import eu.etaxonomy.taxeditor.forms.selectioncomposites.TaxonSelectionComposite;
import eu.etaxonomy.taxeditor.forms.termcombos.LanguageComboComposite;

/**
 * @author nho
 *
 */
public class EditorFormToolkit extends FormToolkit{

	private BoldFontHolder2 boldFontHolder2;

	/**
	 * @param display
	 */
	public EditorFormToolkit(Display display) {
		super(display);
		init();
	}
	
	/**
	 * 
	 */
	private void init() {
		boldFontHolder2 = new BoldFontHolder2();
	}

	/**
	 * @param display
	 */
	public EditorFormToolkit(FormColors colors) {
		super(colors);
		init();
	}

	/**
	 * 
	 * @param parent
	 * @param style
	 * @return
	 */
	public Container createContainer(Composite parent, int style){		
		return new Container(parent, style);
		
	}
	
	/**
	 * 
	 * @param parent
	 * @param labelString
	 * @param textHeight
	 * @param style
	 * @return
	 */
	public MultilineTextWithLabel createMultilineTextWithLabel(Composite parent, String labelString, int textHeight, int style){
		return new MultilineTextWithLabel(parent, labelString, "", textHeight, style);
	}
	
	/**
	 * 
	 * @param parent
	 * @param source
	 * @param style
	 * @return
	 */
	public DescriptionElementSourceComposite createDescriptionElementSourceComposite(Composite parent, DescriptionElementSource source, int style){
		return new DescriptionElementSourceComposite(parent, source, style);
	}
	
	/**
	 * 
	 * @param parent
	 * @param labelString
	 * @param initialText
	 * @param style
	 * @return
	 */
	public TextWithLabelComposite createTextWithLabelComposite(Composite parent, String labelString, String initialText, int style){
		return new TextWithLabelComposite(parent, labelString, initialText, style);
	}
	
	/**
	 * 
	 * @param parent
	 * @param labelString
	 * @param selection
	 * @param style
	 * @return
	 */
	public LanguageComboComposite createLanguageComboComposite(Composite parent, String labelString, Language selection, int style){
		return new LanguageComboComposite(parent, labelString, selection, style);
	}
	
	/**
	 * 
	 * @param parent
	 * @param imageUri
	 * @param style
	 * @return
	 */
	public ImageComposite createImageComposite(Composite parent, URI imageUri, int style){
		return new ImageComposite(parent, imageUri, style);
	}
	
	/**
	 * 
	 * @param parent
	 * @param labelString
	 * @param initialText
	 * @param style
	 * @return
	 */
	public TextActionComposite createTextActionComposite(Composite parent, String labelString, String initialText, int style){
		return new TextActionComposite(parent, labelString, initialText, style);
	}

	/**
	 * 
	 * @param parent
	 * @param annotation
	 * @param style
	 * @return
	 */
	public AnnotationComposite createAnnotationComposite(Composite parent, Annotation annotation, int style) {
		return new AnnotationComposite(parent, annotation, style);
	}
	
	/**
	 * 
	 * @param parent
	 * @param selection
	 * @param style
	 * @return
	 */
	public TaxonSelectionComposite createTaxonSelectionComposite(Composite parent, TaxonBase selection, int style) {
		return new TaxonSelectionComposite(parent, "Taxon", selection, style);
	}
	
	/**
	 * 
	 * @param parent
	 * @param annotatableEntity
	 * @param style
	 * @return
	 */
	public AnnotationSection createAnnotationSection(Composite parent, AnnotatableEntity annotatableEntity,  int style){
		return new AnnotationSection(parent, annotatableEntity, style);
	}
	
	/**
	 * 
	 * @param parent
	 * @param entity
	 * @param style
	 * @return
	 */
	public OriginalSourceSection createOriginalSourceSection(Composite parent, DescriptionElementBase entity, int style){
		return new OriginalSourceSection(parent, entity, style);
	}
	
	
	/**
	 * Creates a section as a part of the form.
	 * 
	 * @param parent
	 *            the section parent
	 * @param sectionStyle
	 *            the section style
	 * @return the section widget
	 */
	public Section adapt(AbstractEditorFormSection section) {
		section.setMenu(section.getParent().getMenu());
		adapt(section, true, true);
		if (section.getToggle() != null) {
			section.getToggle().setHoverDecorationColor(getColors()
					.getColor(IFormColors.TB_TOGGLE_HOVER));
			section.getToggle().setDecorationColor(getColors()
					.getColor(IFormColors.TB_TOGGLE));
		}
		
		section.setFont(boldFontHolder2.getBoldFont(section.getParent().getFont()));

//		if ((section.getStyle() & Section.TITLE_BAR) != 0
//				|| (section.getStyle() & Section.SHORT_TITLE_BAR) != 0) {
			getColors().initializeSectionToolBarColors();
			section.setTitleBarBackground(getColors().getColor(IFormColors.TB_BG));
			section.setTitleBarBorderColor(getColors()
					.getColor(IFormColors.TB_BORDER));
//		}
		// call setTitleBarForeground regardless as it also sets the label color
		section.setTitleBarForeground(getColors()
				.getColor(IFormColors.TB_TOGGLE));
		return section;
	}
	
	private class BoldFontHolder2 {
		private Font normalFont;

		private Font boldFont;

		public BoldFontHolder2() {
		}

		public Font getBoldFont(Font font) {
			createBoldFont(font);
			return boldFont;
		}

		private void createBoldFont(Font font) {
			if (normalFont == null || !normalFont.equals(font)) {
				normalFont = font;
				dispose();
			}
			if (boldFont == null) {
				boldFont = FormFonts.getInstance().getBoldFont(getColors().getDisplay(),
						normalFont);
			}
		}

		public void dispose() {
			if (boldFont != null) {
				FormFonts.getInstance().markFinished(boldFont);
				boldFont = null;
			}
		}
	}


}
