/**
 * 
 */
package eu.etaxonomy.taxeditor.forms.entitysections;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.taxeditor.forms.AbstractEditorFormSection;

/**
 * @author nho
 *
 */
public abstract class AbstractEntitySetSection<T extends VersionableEntity> extends AbstractEditorFormSection {

	protected T entity; 
	protected Composite container;
	
	/**
	 * @param parent
	 * @param style
	 */
	protected AbstractEntitySetSection(Composite parent, T entity, int style) {
		super(parent, style);
		this.entity = entity;
		setTextClient(createToolbar());
		
	}
	
	private Control createToolbar() {
		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);
//		IMenuService menuService = (IMenuService) EditorUtil.getService(IMenuService.class);
//		menuService.populateContributionManager(toolBarManager, "toolbar:de.md.contributions.sectionToolbar");
		
		
		toolBarManager.add(getAddAction());
		
		return toolBarManager.createControl(this);
	}

	/**
	 * @return
	 */
	protected abstract IAction getAddAction();
	
	private void createContainer(){
		if(container != null)
			container.dispose();
		container = toolkit.createContainer(getClient(), SWT.NULL);
		
		container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		container.setLayout(new GridLayout());
	}
	
	/**
	 * Call this method after dynamically changing the client area. 
	 * If the options changed is set to true, will also fire a state changed
	 * event to inform the user of unsaved changes. 
	 * 
	 * @param changed 
	 */
	protected void internalUpdateSection(boolean changed){
		createContainer();
		createDynamicContents();
		this.reflow();
		if(changed)
			firePropertyChangeEvent(null);
	}

	/**
	 * 
	 */
	protected abstract void createDynamicContents();

	/**
	 * Sets this composites entity
	 *
	 * Call this method when the page gets loaded
	 * or otherwise updated from outside
	 *
	 * @param entity the entity to set
	 */
	public void setEntity(T entity) {
		this.entity = entity;
		internalUpdateSection(false);
	}

	/**
	 * @return the entity
	 */
	public T getEntity() {
		return entity;
	}
}
