/**
 * 
 */
package eu.etaxonomy.taxeditor.forms.entitysections;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.common.DescriptionElementSource;
import eu.etaxonomy.taxeditor.forms.AbstractFormComposite;
import eu.etaxonomy.taxeditor.forms.TextWithLabelComposite;
import eu.etaxonomy.taxeditor.forms.selectioncomposites.NameSelectionComposite;
import eu.etaxonomy.taxeditor.forms.selectioncomposites.ReferenceSelectionComposite;

/**
 * @author  nho
 */
public class DescriptionElementSourceComposite extends AbstractFormComposite implements IPropertyChangeListener{

	private DescriptionElementSource source;
	
	private ReferenceSelectionComposite citationSelection;
	private TextWithLabelComposite microCitation;
	private NameSelectionComposite nameSelection;

	public DescriptionElementSourceComposite(Composite parent, DescriptionElementSource source, int style){
		super(parent, style);
		this.source = source;
		
		citationSelection = new ReferenceSelectionComposite(this, "Citation", source.getCitation(), SWT.NULL);
		citationSelection.addPropertyChangeListener(this);
		
		microCitation = new TextWithLabelComposite(this, "Citation Detail", source.getCitationMicroReference(), SWT.NULL);
		microCitation.addPropertyChangeListener(this);
        
        nameSelection = new NameSelectionComposite(this, "Name in Source", source.getNameUsedInSource(), SWT.NULL);
        nameSelection.addPropertyChangeListener(this);
	}
	
	
	/**
	 * @return the source
	 */
	public DescriptionElementSource getSource() {
		return source;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		source.setCitation(citationSelection.getSelection());
		source.setCitationMicroReference(microCitation.getText());
		source.setNameUsedInSource(nameSelection.getSelection());
		firePropertyChangeEvent(event);
	}	
}