/**
 * 
 */
package eu.etaxonomy.taxeditor.forms.entitysections;

import java.util.Set;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.model.common.DescriptionElementSource;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;

/**
 * @author n.hoffmann
 *
 */
public class OriginalSourceSection extends AbstractEntitySetSection<DescriptionElementBase> implements IPropertyChangeListener{


	public OriginalSourceSection(Composite parent, DescriptionElementBase entity, int style){
		super(parent, entity, Section.TWISTIE | Section.COMPACT | style);
		
		setText("Citations");
	}

	/*
	 * 
	 */
	public void createDynamicContents() {
		
		Set<DescriptionElementSource> sources = null;
		if(getEntity() != null){
			sources = getEntity().getSources();
		}
		
		if(sources == null || sources.size() == 0){
			toolkit.createLabel(container, "No citations yet.");
		}else{
			for(DescriptionElementSource source : sources){
				DescriptionElementSourceComposite descriptionElementSourceComposite = toolkit.createDescriptionElementSourceComposite(container, source, SWT.NULL);
				descriptionElementSourceComposite.addPropertyChangeListener(this);
				
				Button button_remove = toolkit.createButton(container, "Remove", SWT.PUSH);
				button_remove.addSelectionListener(new RemoveListener(source));
			}
		}
	}
	
	private class RemoveListener extends SelectionAdapter{
		private DescriptionElementSource source;
		
		public RemoveListener(DescriptionElementSource source){
			this.source = source;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			getEntity().removeSource(source);
			internalUpdateSection(true);
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractEntitySetSection#getAddAction()
	 */
	@Override
	protected IAction getAddAction() {
		Action addCitation = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				getEntity().addSource(DescriptionElementSource.NewInstance());
				if(! getSection().isExpanded())
					getSection().setExpanded(true);
				internalUpdateSection(true);
			}
		};
		addCitation.setToolTipText("Create a new citation");
		return addCitation;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		firePropertyChangeEvent(event);
	}	
}
