// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.tabbed;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.reference.INomenclaturalReference;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.Page;

/**
 * @author n.hoffmann
 * @created 05.06.2009
 * @version 1.0
 */
public class NomRefPropertySection extends AbstractNamePropertySection {
	private static final Logger logger = Logger
			.getLogger(NomRefPropertySection.class);
	
	private INomenclaturalReference reference;
	private String microReference;

	
	private CCombo referenceTypeCombo;
	
	private Text titleText;
	private Text placePublishedText;
	private Text datePublishedText;
	private Text editorText;
	private Text publisherText;
	private Text inSeriesText;
	private Text pagesText;
	private Text seriesPartText;
	private Text volumeText;
	private Text isbnText;
	private Text editionText;
	
	
	private Text microReferenceText;

	private Button searchButton;

    protected ModifyListener listener = new ModifyListener() {
        
        public void modifyText(ModifyEvent arg0) {
        	logger.warn("element modified");
        	
        	reference.setTitleCache(cacheText.getText());
        	
        	// TODO refresh the editor
//        	EditorUtil.getActiveEditorPage(Page.NAME).setDirty();
        }
    };
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#createControls(org.eclipse.swt.widgets.Composite, org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage)
	 */
	@Override
	public void createControls(Composite parent, EditorTabbedPropertySheetPage tabbedPropertySheetPage) {
		
        super.createControls(parent, tabbedPropertySheetPage);
        
        CLabel empty = getWidgetFactory().createCLabel(composite, "");
        searchButton = createSearchButton(composite);
        
        /*
        CLabel referenceTypeLabel = getWidgetFactory().createCLabel(composite, "Reference Type");
        referenceTypeCombo = getWidgetFactory().createCCombo(composite);
        referenceTypeCombo.setLayoutData(layoutData);
        
        for (Class clazz : CdmStore.getReferenceTypes().keySet()){
        	referenceTypeCombo.add(CdmStore.getReferenceTypes().get(clazz));
        }
        
        CLabel titleLabel = getWidgetFactory().createCLabel(composite, "Title");
        titleText = getWidgetFactory().createText(composite, "");
        titleText.setLayoutData(layoutData);
        
        CLabel placePublishedLabel = getWidgetFactory().createCLabel(composite, "Place Published");
        placePublishedText = getWidgetFactory().createText(composite, "");
        placePublishedText.setLayoutData(layoutData);
        
        CLabel datePublishedLabel = getWidgetFactory().createCLabel(composite, "Date Published");
        datePublishedText = getWidgetFactory().createText(composite, "");
        datePublishedText.setLayoutData(layoutData);
        
        CLabel editorLabel = getWidgetFactory().createCLabel(composite, "Editor");
        editorText = getWidgetFactory().createText(composite, "");
        editorText.setLayoutData(layoutData);
        
        CLabel publisherLabel = getWidgetFactory().createCLabel(composite, "Publisher");
        publisherText = getWidgetFactory().createText(composite, "");
        publisherText.setLayoutData(layoutData);
        
        CLabel inSeriesLabel = getWidgetFactory().createCLabel(composite, "In Series");
        inSeriesText = getWidgetFactory().createText(composite, "");
        inSeriesText.setLayoutData(layoutData);
        
        CLabel pagesLabel = getWidgetFactory().createCLabel(composite, "Pages");
        pagesText = getWidgetFactory().createText(composite, "");
        pagesText.setLayoutData(layoutData);
        
        CLabel seriesPartLabel = getWidgetFactory().createCLabel(composite, "Series Part");
        seriesPartText = getWidgetFactory().createText(composite, "");
        seriesPartText.setLayoutData(layoutData);
        
        CLabel volumeLabel = getWidgetFactory().createCLabel(composite, "Volume");
        volumeText = getWidgetFactory().createText(composite, "");
        volumeText.setLayoutData(layoutData);
        
        CLabel isbnLabel = getWidgetFactory().createCLabel(composite, "ISBN");
        isbnText = getWidgetFactory().createText(composite, "");
        isbnText.setLayoutData(layoutData);
        
        CLabel editionLabel = getWidgetFactory().createCLabel(composite, "Edition");
        editionText = getWidgetFactory().createText(composite, "");
        editionText.setLayoutData(layoutData);
        
        */
        CLabel microReferenceLabel = getWidgetFactory().createCLabel(composite, "Reference Detail");
        microReferenceText = getWidgetFactory().createText(composite, "");
        microReferenceText.setLayoutData(layoutData);
        
        
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#refresh()
	 */
	@Override
	public void refresh() {
		super.refresh();
		
		section.setText("Nomenclatural Reference");
		
		if(reference != null){
			cacheText.removeModifyListener(listener);
			cacheText.setText(CdmUtils.Nz(reference.getTitleCache()));
			cacheText.addModifyListener(listener);
	
			editCacheButton.setSelection(reference.isProtectedTitleCache());
			cacheText.setEnabled(reference.isProtectedTitleCache());
		}
		
		microReferenceText.setText(CdmUtils.Nz(microReference));
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#setInput(org.eclipse.ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void setInput(IWorkbenchPart part, ISelection selection) {
        super.setInput(part, selection);
        reference = name.getNomenclaturalReference();
        microReference = name.getNomenclaturalMicroReference();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.propertysheet.tabbed.AbstractNamePropertySection#toggleCacheEditing()
	 */
	@Override
	protected void toggleCacheEditing() {
		if(reference.isProtectedTitleCache()){
			reference.setProtectedTitleCache(false);
		}else{
			reference.setProtectedTitleCache(true);
		}
		
		super.toggleCacheEditing();
	}
}
