// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.datasource.wizard;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.database.CdmPersistentDataSource;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;


/**
 * @author n.hoffmann
 * @created 19.05.2009
 * @version 1.0
 */
public abstract class CdmDataSourceCredentialsWizardPage extends WizardPage implements ModifyListener{

	private ICdmDataSource dataSource;
	
	protected Text text_password;
	protected Text text_databaseName;
	protected Text text_username;
	
	protected Group authenticationGroup;
	protected Group locationGroup;
	protected Group nomenclaturalCodeGroup;

	protected Composite composite;

	protected Composite parent;

	protected String database;
	protected String username;
	protected String password;
	
	protected NomenclaturalCode nomenclaturalCode;
	
	/**
	 * @param pageName
	 */
	protected CdmDataSourceCredentialsWizardPage(String pageName) {
		super(pageName);
		this.setPageComplete(false);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		this.parent = parent;
		
		// Create top-level composite 
		
		composite = new Composite(parent, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		GridLayout formLayout = new GridLayout();
		formLayout.numColumns = 2;
		composite.setLayout(formLayout);

		// create a database specific form
		createDatabaseForm();
		
		// create the authentication input fields
		createAuthenticationForm();
		
		// create nomenclatural code combo
		createNomenclaturalCodeForm();
		
		// Create composite for buttons
		Composite buttonComposite = new Composite(composite, SWT.NONE);
		buttonComposite.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false));
		GridLayout buttonLayout = new GridLayout();
		buttonLayout.numColumns = 2;
		buttonComposite.setLayout(buttonLayout);
		
		// Create test connection button
		Button testButton = new Button(buttonComposite, SWT.NONE);
		testButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
		testButton.setText("Test connection");
		
		// Test connection when button is pressed
		testButton.addSelectionListener(new SelectionAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				testDbConfiguration();
			}
		});
		
		setControl(composite);
		
		init();
	}

	protected void createAuthenticationForm(){
		// Create group composite for authentication data
		authenticationGroup = new Group(composite, SWT.NONE);
		authenticationGroup.setText("Authentication");
		authenticationGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		GridLayout authenticationLayout = new GridLayout();
		authenticationLayout.numColumns = 2;
		authenticationGroup.setLayout(authenticationLayout);
		
		// Create database name label
		Label databaseNameLabel = new Label(authenticationGroup, SWT.NONE);
		databaseNameLabel.setText("Database Name:");

		// Create database name input
		text_databaseName = new Text(authenticationGroup, SWT.BORDER);
		text_databaseName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		text_databaseName.addModifyListener(this);

		// Create username label
		Label usernameLabel = new Label(authenticationGroup, SWT.NONE);
		usernameLabel.setText("User Name:");
		
		// Create username input
		text_username = new Text(authenticationGroup, SWT.BORDER);
		text_username.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		text_username.addModifyListener(this);
		
		// Create password label
		Label passwordLabel = new Label(authenticationGroup, SWT.NONE);
		passwordLabel.setText("Password:");

		// Create password input
		text_password = new Text(authenticationGroup, SWT.BORDER);
		text_password.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		text_password.addModifyListener(this);
	}
	
	/**
	 * 
	 */
	private void createNomenclaturalCodeForm() {
		nomenclaturalCodeGroup = new Group(composite , SWT.NONE);
		nomenclaturalCodeGroup.setLayout(new GridLayout());
		
		nomenclaturalCode = dataSource != null ? dataSource.getNomenclaturalCode() : PreferencesUtil.getPreferredNomenclaturalCode();
		
		for (final NomenclaturalCode code : NomenclaturalCodeHelper.getSupportedCodes()) {
			Button button = new Button(nomenclaturalCodeGroup, SWT.RADIO);
			button.setText(NomenclaturalCodeHelper.getDescription(code));
			button.setData(code);
			if (nomenclaturalCode != null) {
				button.setSelection(nomenclaturalCode.equals(code));
			}
			button.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					nomenclaturalCode = (NomenclaturalCode) e.widget.getData();
				}
			});
		}		
	}
	
	
	
	/**
	 * Tries to open a connection to the given dataSource. Generates a message on either 
	 * failure or success 
	 * 
	 * @param dataSource
	 */
	public void testDbConfiguration(){
		try{
			getDataSource().testConnection();
			MessageDialog.openConfirm(parent.getShell(), "Test successful", "Test successful!");
		} catch(Exception e){
			MessageDialog.openWarning(parent.getShell(), "Test unsuccessful", "Test unsuccessful! \n\nSee error.log for details.");
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Form implementation for the specific database
	 */
	public abstract void createDatabaseForm(); 
	
	public abstract void updateLocation();
	
	public abstract void updateDataSource();
	
	public void checkPageComplete(){
		boolean complete = database.length() != 0;
//		complete &= username.length() != 0;
		
		this.setPageComplete(complete);
	}
	
	/**
	 * Initialize text fields 
	 */
	public void init() {
		if(getDataSource() != null){
			modifyTextWithoutTriggeringListeners(text_databaseName, this, getDataSource().getDatabase());
			modifyTextWithoutTriggeringListeners(text_username, this, getDataSource().getUsername());
			modifyTextWithoutTriggeringListeners(text_password, this, getDataSource().getPassword());
		}
	}
	
	/**
	 * updates the current datasource with form values
	 */
	public void updateAuthentication(){
		
		database = text_databaseName.getText();
		username = text_username.getText();
		password = text_password.getText();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.WizardPage#getNextPage()
	 */
	@Override
	public IWizardPage getNextPage() {
		return null;
	}

	/**
	 * @param dataSource the dataSource to set
	 */
	public void setDataSource(ICdmDataSource dataSource) {
		this.dataSource = dataSource;
	}

	/**
	 * @return the dataSource
	 */
	public ICdmDataSource getDataSource() {
		return dataSource;
	}
	
	/**
	 * @return
	 */
	public String getDataSourceName() {
		if(dataSource instanceof CdmPersistentDataSource){
			return ((CdmPersistentDataSource) dataSource).getName();
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		updateLocation();
		updateAuthentication();
		updateDataSource();
		checkPageComplete();
	}	
	
	/**
	 * 
	 * @param text	
	 * @param listener
	 * @param string
	 */
	protected void modifyTextWithoutTriggeringListeners(Text text, ModifyListener listener, String string){
		text.removeModifyListener(listener);
		text.setText(string);
		text.addModifyListener(listener);
	}
}
