// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.io.common.CdmDefaultExport;
import eu.etaxonomy.cdm.io.common.IExportConfigurator;
import eu.etaxonomy.cdm.io.jaxb.JaxbExportConfigurator;
import eu.etaxonomy.cdm.io.jaxb.JaxbImportConfigurator;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author n.hoffmann
 * @created Sep 11, 2009
 * @version 1.0
 */
public class ExportHandler extends AbstractIOHandler{
	private static final Logger logger = Logger
			.getLogger(ExportHandler.class);

	private ExportHandler(CdmApplicationController applicationController){
		super(applicationController);
	}
	
	public static ExportHandler NewInstance(CdmApplicationController applicationController){
		return new ExportHandler(applicationController);
	}
	
	public void doExport(final IExportConfigurator configurator){
		Assert.isNotNull(configurator, "Configuration may not be null");
		
		// terminate any open transactions
		IConversationEnabled activePart = (IConversationEnabled) StoreUtil.getActivePage().getActivePart();
		activePart.getConversationHolder().commit(false);
		
		
		CdmDefaultExport<IExportConfigurator> exporter = new CdmDefaultExport<IExportConfigurator>();

		exporter.setCdmAppController(applicationController);
		
		
		try{
			exporter.invoke(configurator);
		}catch(RuntimeException e){
			StoreUtil.errorDialog("Error exporting data", "An error occured while" +
					"exporting to destination '" + configurator.getDestinationNameString() + "'.\n" +
							"Please check error log for details.");
			logger.error("Error exporting data", e);
		}
		
		// restarting transaction and committing it to trigger change listener
		// TODO verify correct behaviour
		activePart.getConversationHolder().startTransaction();
		activePart.getConversationHolder().commit();
	}
	
	/**
	 * @param jaxb
	 * @return
	 */
	private JaxbExportConfigurator getConfigurator(TYPE type) {
		Assert.isNotNull(type, "Type should not be null");
				
		switch (type){
			case Jaxb:
				return JaxbExportConfigurator.NewInstance(null, null);
			default:
				StoreUtil.notImplementedMessage();
				throw new IllegalArgumentException("Export not supported yet");
		}
	}
	
	public final JaxbExportConfigurator JaxbConfigurator() { return (JaxbExportConfigurator) getConfigurator(TYPE.Jaxb);	}


}
