/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.util.Vector;

import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeEvent;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;

/**
 * Exchangeable algorithm defining what should happen after a client receives a notification
 * that data was changed
 * 
 * @author n.hoffmann
 * @created 01.04.2009
 * @version 1.0
 */
public interface IDataChangeBehavior {

	/**
	 * If a DataChangeEvent was considered relevant for the calling object
	 * the instructions implemented in this method will get executed
	 */
	public void reactOnDataChange(CdmDataChangeMap changeEvents);
	
	/**
	 * Define what should happen when an insert occurs. Clients may do 
	 * further checking to see if the insert really affects their data 
	 * 
	 * @param source
	 */
	public void reactOnInsert(Vector<CdmDataChangeEvent> events);
	
	/**
	 * Define what should happen when an update occurs. Clients may do 
	 * further checking to see if the update really affects their data 
	 * 
	 * @param source
	 */
	public void reactOnUpdate(Vector<CdmDataChangeEvent> events);
	
	/**
	 * Define what should happen when an delete occurs. Clients may do 
	 * further checking to see if the delete really affects their data 
	 * 
	 * @param source
	 */
	public void reactOnDelete(Vector<CdmDataChangeEvent> events);

	
	
}
