/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;

/**
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 18.03.2009
 * @version 1.0
 */
public class NameHelper {	

	@SuppressWarnings("unchecked")
	public static String getDisplayName(TaxonNameBase name){
		return name.getTitleCache();
	}
	
	public static String getDisplayName(TaxonBase<?> taxon){
		return getDisplayName(taxon.getName());
	}
	
	@SuppressWarnings("unchecked")
	public static String getDisplayNameWithRef(TaxonBase taxonBase) {
		TaxonNameBase name = taxonBase.getName();
		if (name != null) {
			name = HibernateProxyHelper.deproxy(name, TaxonNameBase.class);
			if (name.getFullTitleCache() == null || name.getFullTitleCache().length() == 0) {
				return CdmUtils.Nz(name.getTitleCache());
			} else {
				return name.getFullTitleCache();
			}
		}
		return "";
	}
	
	/**
	 * Checks whether name is the basionym for ALL names 
	 * 	in its group.
	 * @param name
	 * 
	 * @return
	 */	
	@SuppressWarnings("unchecked")
	public static boolean isNameGroupBasionym(TaxonNameBase name) {
		if (name == null) {
			return false;
		}
		return name.isGroupsBasionym();		
	}
	

	/**
	 * @param name
	 * @return
	 */
	public static boolean isNameSupraSpecific(TaxonNameBase<?, ?> name) {
		if (name == null || name.getRank() == null) {
			return false;
		}
		return name.getRank().isHigher(Rank.SPECIES());
	}
	
	/**
	 * Checks whether name belongs to the same homotypic group as taxon's name.
	 * 
	 * @param name
	 * @param taxon
	 * @return
	 */
	public static boolean isNameHomotypic(TaxonNameBase<?, ?> name, Taxon taxon) {
		return name.isHomotypic(taxon.getName());
	}
	
}
