/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Taxon;

/**
 * @author p.ciardelli
 * @created 05.02.2009
 * @version 1.0
 * @author n.hoffmann
 */
public class CreateTaxonDescriptionOperation extends AbstractPostOperation {
	
	private TaxonDescription description;
	private boolean isImageGallery;

	public CreateTaxonDescriptionOperation(String label, IUndoContext undoContext,
			Taxon taxon, IPostOperationEnabled postOperationEnabled) {
		this(label, undoContext, taxon, postOperationEnabled, false);
	}

	public CreateTaxonDescriptionOperation(String label, IUndoContext undoContext,
			Taxon taxon, IPostOperationEnabled postOperationEnabled, boolean isImageGallery) {
		super(label, undoContext, taxon, postOperationEnabled);
		
		this.isImageGallery  = isImageGallery;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		description = TaxonDescription.NewInstance(taxon);
		description.setImageGallery(isImageGallery);
		
		return postExecute(description);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		taxon.addDescription(description);
		
		return postExecute(description);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		taxon.removeDescription(description);
		
		return postExecute(null);
	}
}
