// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store;

import java.util.HashSet;
import java.util.Set;
import java.util.SortedSet;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.ExtensionType;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.OrderedTermBase;
import eu.etaxonomy.cdm.model.common.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;

/**
 * All vocabularies may be accessed through this store.
 * Note: This is for internal use. When using vocabularies in the UI, try to use the preferred terms from
 * PreferenceUtil
 * 
 * @author n.hoffmann
 * @created 25.06.2009
 * @version 1.0
 */
public class VocabularyStore {
	
	/**
	 * 
	 * @return
	 */
	public static SortedSet<SpecimenTypeDesignationStatus> getSpecimenTypeDesignationStatus() {
		SortedSet specimenTypeDesignationStatus = getTermVocabulary(VocabularyEnum.SpecimenTypeDesignationStatus).getTermsOrderedByLabels(CdmStore.getDefaultLanguage());
		return specimenTypeDesignationStatus;
	}

	/**
	 * 
	 * @return
	 */
	public static SortedSet<SpecimenTypeDesignationStatus> getNameTypeDesignationStatus() {
		SortedSet nameTypeDesignationStatus = getTermVocabulary(VocabularyEnum.NameTypeDesignationStatus).getTermsOrderedByLabels(CdmStore.getDefaultLanguage());
		return nameTypeDesignationStatus;
	}
	
	/**
	 * 
	 * @return
	 */
	public static SortedSet<TaxonRelationshipType> getTaxonRelationshipTypes(){
		SortedSet taxonRelationshipTypes = getOrderedTermVocabulary(VocabularyEnum.TaxonRelationshipType).getOrderedTerms();
		return taxonRelationshipTypes;
	}
	
	/**
	 * 
	 * @return
	 * 
	 * @deprecated handle via preferences
	 */
	public static SortedSet<TaxonRelationshipType> getConceptRelationshipTypes() {
		SortedSet<TaxonRelationshipType> conceptRelationshipTypes = getTaxonRelationshipTypes();
		// remove these two relations as they are considered standard taxon relations
		conceptRelationshipTypes.remove(TaxonRelationshipType.MISAPPLIED_NAME_FOR());
		conceptRelationshipTypes.remove(TaxonRelationshipType.TAXONOMICALLY_INCLUDED_IN());
	
		return conceptRelationshipTypes;
	}
	
	/**
	 * 
	 * @return
	 */
	public static Set<Language> getLanguages(){

		Set languages = getTermVocabulary(VocabularyEnum.Language).getTerms();
		
//		SortedSet languages = getDefault().getTermVocabulary(VocabularyEnum.Language).getTermsOrderedByLabels(getDefaultLanguage());
		return languages;
	}
	
	public static SortedSet<? extends DefinedTermBase> getTerms(VocabularyEnum vocabularyType){
		
		TermVocabulary<DefinedTermBase> terms = getTermVocabulary(vocabularyType);
		
		if(terms != null) {
			return terms.getTermsOrderedByLabels(CdmStore.getDefaultLanguage());
		}
		return null;
	}
	
	/**
	 * @param selected
	 */
	public static void delete(DefinedTermBase selected) {
		CdmStore.getTermService().delete(selected);
	}
	/**
	 * 
	 * @return
	 */
	public static SortedSet<Feature> getFeatures() {
		SortedSet features = getTermVocabulary(VocabularyEnum.Feature).getTermsOrderedByLabels(CdmStore.getDefaultLanguage());
		features.remove(Feature.IMAGE());
		return features;

	}
	
	/**
	 * 
	 * @return
	 */
	public static SortedSet<NameRelationshipType> getNameRelationshipTypes() {
		SortedSet nameRelationshipTypes = getTermVocabulary(VocabularyEnum.NameRelationshipType).getTermsOrderedByLabels(CdmStore.getDefaultLanguage());
		return nameRelationshipTypes;
	}
	
	/**
	 * 
	 * @return
	 */
	public static Set<NomenclaturalStatusType> getNomenclaturalStatusTypes(){
		// TODO sort types
		Set nomenclaturalStatusTypes = getTermVocabulary(VocabularyEnum.NomenclaturalStatusType).getTermsOrderedByLabels(CdmStore.getDefaultLanguage());
		return nomenclaturalStatusTypes;
	}
	

	/**
	 * @return 
	 * 
	 */
	public static Set<MarkerType> getNonTechnicalMarkerTypes() {
		Set<MarkerType> nonTechnicalMarkerTypes = new HashSet<MarkerType>();
		for (Object type : getTermVocabulary(VocabularyEnum.MarkerType).getTermsOrderedByLabels(CdmStore.getDefaultLanguage())) {
			if (((MarkerType) type).isTechnical() == false) {
				nonTechnicalMarkerTypes.add((MarkerType) type);
			}
		}
		
		return nonTechnicalMarkerTypes;
	}
	

	/**
	 * @return
	 */
	public static Set<ExtensionType> getExtensionTypes() {
		Set extensionTypes = getTermVocabulary(VocabularyEnum.ExtensionType).getTermsOrderedByLabels(CdmStore.getDefaultLanguage());
		return extensionTypes;
	}
	
	/**
	 * Retrieve all ranks from data store.
	 * 
	 * @return
	 */
	public static SortedSet<Rank> getRanks(){
		SortedSet ranks = getOrderedTermVocabulary(VocabularyEnum.Rank).getOrderedTerms();
		return ranks;
	}
	
	
	/**
	 * Retrieve all presence and absence terms from data store.
	 *  
	 * @return
	 */
	public static SortedSet<PresenceAbsenceTermBase<PresenceAbsenceTermBase<?>>> getPresenceAbsenceTerms() {
		SortedSet presenceAbsenceTerms = getTermVocabulary(VocabularyEnum.PresenceTerm).getTermsOrderedByLabels(CdmStore.getDefaultLanguage());
			presenceAbsenceTerms.addAll(getTermVocabulary(VocabularyEnum.AbsenceTerm).getTermsOrderedByLabels(CdmStore.getDefaultLanguage()));

		return presenceAbsenceTerms;
	}

	
	/*************** save methods ******************************/
	/**
	 * Adds a term to a vocabulary
	 * 
	 * @param feature
	 */
	public static void saveTerm(DefinedTermBase term, VocabularyEnum vocabularyType) {
		TermVocabulary<DefinedTermBase> vocabulary = getTermVocabulary(vocabularyType);		
		vocabulary.addTerm(term);
		
		CdmStore.getVocabularyService().save(vocabulary);
	}
	
	/**
	 * Save a vacabulary to data store
	 * 
	 * @param feature
	 */
	public static void updateVocabulary(DefinedTermBase term) {
		CdmStore.getTermService().save(term);
	}
	
	
	
	/*************** internal methods **************************/
	
	private static Language getDefaultLanguage(){
		return CdmStore.getDefaultLanguage();
	}
	
	private static OrderedTermVocabulary<? extends OrderedTermBase> getOrderedTermVocabulary(VocabularyEnum vocabularyType){
 		TermVocabulary<DefinedTermBase> vocabulary = getTermVocabulary(vocabularyType);		
 		return HibernateProxyHelper.deproxy(vocabulary, OrderedTermVocabulary.class);
	}
	
	public static TermVocabulary<DefinedTermBase> getTermVocabulary(VocabularyEnum vocabularyType){
		return CdmStore.getVocabularyService().getVocabulary(vocabularyType);
	}

}
